﻿

Namespace Charting
    Partial Class Legend2D

        ''' <summary>
        ''' Helper class for accessing the properties of a logical row in a Legend2D
        ''' </summary>
        Public Class LegendRow
            Private ReadOnly m_Legend As ILegend
            Private ReadOnly m_Row As Integer

            ''' <summary>
            ''' Create a new LegendRow
            ''' </summary>
            ''' <param name="legend">The associated Legend2D instance</param>
            ''' <param name="row">1-based index of the row</param>
            ''' <remarks>
            ''' The LegendRow consructor is typically only used by the Legend2D object itself
            ''' </remarks>
            Public Sub New(legend As ILegend, row As Integer)
                m_Legend = legend
                m_Row = row
            End Sub

            ''' <summary>
            ''' Returns true if any series for this row are visible, false otherwise
            ''' </summary>
            Public Property Visible As Boolean
                Get
                    Return m_Legend.RowVisible(m_Row)
                End Get
                Set(value As Boolean)
                    m_Legend.RowVisible(m_Row) = value
                End Set
            End Property

            Public Property Caption As String
                Get
                    Return m_Legend.RowCaption(m_Row)
                End Get
                Set(value As String)
                    m_Legend.RowCaption(m_Row) = value
                End Set
            End Property

            Public Property Color As Color
                Get
                    Return m_Legend.RowColor(m_Row)
                End Get
                Set(value As Color)
                    m_Legend.RowColor(m_Row) = value
                End Set
            End Property
        End Class
    End Class
End Namespace
