﻿Namespace Charting
    ''' <summary>
    ''' Helper class for measuring how wide controls need to be based on the measured size of strings
    ''' </summary>
    ''' <remarks>
    ''' Once this class is created it can be presented a series of controls.  Each control is examined
    ''' and the largest Size needed to hold any of them is accumulated.  At any point the Width or
    ''' Height properties can be accessed to get the max size seend so far.  As a convenience, this
    ''' class also provides HeightMargin and WidthMargin properties.  The values returned from the
    ''' Height and Width properties includes margins on both sizes (left/right or top/bottom).
    ''' </remarks>
    Public Class MeasureString
        Private m_Size As New Size

        Private ReadOnly m_HeightMargin As Integer
        Private ReadOnly m_WidthMargin As Integer

        Public Sub New(Optional widthMargin As Integer = 0, Optional heightMargin As Integer = 0)
            m_HeightMargin = heightMargin
            m_WidthMargin = widthMargin
        End Sub

        Public ReadOnly Property Width As Integer
            Get
                Return CInt(Math.Ceiling(m_Size.Width)) + 2 * m_WidthMargin
            End Get
        End Property

        Public ReadOnly Property Height As Integer
            Get
                Return CInt(Math.Ceiling(m_Size.Height)) + 2 * m_HeightMargin
            End Get
        End Property

        Public Sub Examine(control As Control)
            Examine(control, control.Text, control.Font)
        End Sub

        Public Sub Examine(control As Control, text As String, font As Font)
            Using g As Graphics = control.CreateGraphics()
                Dim size As SizeF = g.MeasureString(text, font)
                If size.Height > m_Size.Height Then m_Size.Height = CInt(size.Height)
                If size.Width > m_Size.Width Then m_Size.Width = CType(size.Width, Integer)
            End Using
        End Sub
    End Class
End Namespace