﻿
Imports Steema.TeeChart
Imports Steema.TeeChart.Styles

Namespace Charting
    Public Class TeeChartLegend
        Inherits Legend2D

        Private m_Chart As TChart

        Public Property CenterLegend() As Boolean = True

        Public Overloads Sub Initialize(chart As TChart, rowLabels As IList(Of String), columnLabels As IList(Of String), rowColors As IList(Of Color))
            m_Chart = chart

            Dim basicWidth As Integer = Me.PreferredViewWidth
            Const extraMargin As Integer = 16

            chart.Legend.Visible = False
            chart.Panel.MarginUnits = Steema.TeeChart.PanelMarginUnits.Pixels
            chart.Panel.MarginRight = basicWidth + (extraMargin * 2)
            MyBase.Initialize(rowLabels, columnLabels, rowColors)
            Me.Anchor = AnchorStyles.Right Or AnchorStyles.Top
            AddHandler chart.Resize, New EventHandler(AddressOf ResizeHandler)
            chart.Controls.Add(Me)
            Me.Width = basicWidth
            Me.Height = Me.PreferredViewHeight
            Me.Left = chart.Right - (basicWidth + extraMargin)
            PositionLegendTop()
            Me.Refresh()
        End Sub

        Private Sub ResizeHandler(ByVal sender As Object, ByVal e As EventArgs)
            Try
                MyBase.Resize()
            Catch ex As Exception
                ErrorsFromGUI(, ex)
            End Try
        End Sub

        Public Overloads Sub Refresh()
            MyBase.Refresh()
            If Not m_IsInitialized Then Return
            UpdateChart()
        End Sub

        Public Sub SyncVisibility()
            Dim changeCount As Integer = 0
            For Each series As FastLine In m_Chart.Series
                If TypeOf series.Tag Is LegendSeries Then
                    Dim info As LegendSeries = CType(series.Tag, LegendSeries)

                    If series.Visible <> Me.Cell(info.RowIndex, info.ColumnIndex).Visible Then
                        Me.Cell(info.RowIndex, info.ColumnIndex).Visible = series.Visible
                        changeCount += 1
                    End If
                End If
            Next

            If changeCount > 0 Then
                RefreshViewPanel()
            End If
        End Sub

        Protected Overrides Sub UpdateChart()
            Dim columnThickness(MyBase.ColumnCount) As Integer
            Dim thickness As Integer = 0
            For ixColumn As Integer = 1 To MyBase.ColumnCount
                If ColumnVisible(ixColumn) Then
                    thickness += 1
                    If thickness > MaxLineThickness Then thickness = 1
                    columnThickness(ixColumn) = thickness
                End If
            Next

            Dim highlightRow As Integer = MyBase.HighlightedRow
            Dim highlightColumn As Integer = MyBase.HighlightedColumn

            For Each series As FastLine In m_Chart.Series
                If TypeOf series.Tag Is LegendSeries Then
                    Dim info As LegendSeries = CType(series.Tag, LegendSeries)
                    series.Color = Me.Row(info.RowIndex).Color
                    series.LinePen.Width = columnThickness(info.ColumnIndex)

                    If highlightRow > 0 Then
                        series.Visible = info.RowIndex = highlightRow AndAlso Me.Cell(info.RowIndex, info.ColumnIndex).Visible
                    ElseIf highlightColumn > 0 Then
                        series.Visible = info.ColumnIndex = highlightColumn AndAlso Me.Cell(info.RowIndex, info.ColumnIndex).Visible
                    Else
                        series.Visible = Me.Cell(info.RowIndex, info.ColumnIndex).Visible
                    End If
                End If
            Next
        End Sub

        Protected Overrides Sub ResizeForEditing(fullSize As Boolean)
            Me.Height = PreferredEditHeight
            EditPanel.Height = Me.Height - ShadowThickness
            Dim newWidth As Integer = CInt(IIf(fullSize, m_Chart.Width * 0.6, PreferredEditWidth))
            Dim oldWidth As Integer = Me.Width
            Me.Width = newWidth
            Me.Left -= Me.Width - oldWidth
            PositionLegendTop()
        End Sub

        Private Sub PositionLegendTop()
            If CenterLegend Then
                Me.Top = (m_Chart.Height - Me.PreferredViewHeight) \ 2
            Else
                Dim t As Integer = m_Chart.Axes.Left.AxisRect().Top
                Me.Top = CInt(IIf(t = 0, 29, t))
                m_Chart.SubHeader.Top = Me.Bottom + 6
            End If
        End Sub

        Protected Overrides Sub ResizeForViewing()
            Me.Height = PreferredViewHeight + ShadowThickness
            ViewPanel.Height = Me.Height - ShadowThickness
            Dim tempWidth As Integer = Me.Width
            Me.Width = PreferredViewWidth + ShadowThickness
            Me.Left -= Me.Width - tempWidth
            PositionLegendTop()
        End Sub
    End Class
End Namespace
