﻿Imports Steema.TeeChart.Drawing
Imports Steema.TeeChart

Namespace Charting

    ''' <summary>
    ''' Low-level wrapper for TeeChart to apply across-the-board customizations needed for FUTURE
    ''' </summary>
    Public Class SerafimTeeChart
        Inherits TChart

        Private m_Legend As TeeChartLegend
        Private WithEvents UnzoomButton As Button

        Public Sub EnableCustomLegend(rowLabels As IList(Of String), columnLabels As IList(Of String), rowColors As IList(Of Color))
            If rowLabels Is Nothing OrElse rowLabels.Count = 0 Then Return
            If columnLabels Is Nothing OrElse columnLabels.Count = 0 Then Return
            If rowColors Is Nothing OrElse rowColors.Count = 0 Then Return

            m_Legend = New TeeChartLegend()
            m_Legend.Initialize(Me, rowLabels, columnLabels, rowColors)
        End Sub

        Public Overloads Sub Refresh()
            MyBase.Refresh()
            If Not m_Legend Is Nothing Then
                m_Legend.Refresh()
            End If
        End Sub

        Public Sub EnsureLegendVisible()
            If Not m_Legend Is Nothing Then
                Me.Legend.Visible = False
                m_Legend.Visible = True
            End If
        End Sub

        Public Sub SyncVisibility()
            If m_Legend IsNot Nothing Then
                m_Legend.SyncVisibility()
            End If
        End Sub

        ''' <summary>
        ''' Constructor applies default property values needed for FUTURE
        ''' </summary>
        Sub New()

            'Assign default values
            With Me
                'These properties have default values that have changed since TeeChart 2

                'Disable 3D appearance
                .Aspect.View3D = False

                'Turn off gradients
                .Panel.Gradient.Visible = False
                .Walls.Back.Gradient.Visible = False

                'Adjust default colors
                .Walls.Back.Color = .Panel.Color

                'Adjust fonts
                .Header.Font.Brush.Color = Color.Blue
                .Header.Font.Name = "Arial"
                .Header.Font.Size = 10
                .Header.Font.SizeFloat = 10.0!
                .Axes.Left.Labels.Font.Name = "Arial"
                .Axes.Right.Labels.Font.Name = "Arial"
                .Axes.Top.Labels.Font.Name = "Arial"
                .Axes.Bottom.Labels.Font.Name = "Arial"
                .Axes.Left.Title.Font.Name = "Arial"
                .Axes.Right.Title.Font.Name = "Arial"
                .Axes.Top.Title.Font.Name = "Arial"
                .Axes.Bottom.Title.Font.Name = "Arial"

                'This fixes the problem with axis labels overlapping the axis title
                .Axes.Left.FixedLabelSize = False

                'The root cause of the TeeChart errors subsequent to v2 was a default value of
                'OptimizedBuffer for the BufferStyle property.  This is a workaround as decribed in
                'http://www.teechart.net/support/viewtopic.php?f=4&t=14156#p62249
                .Graphics3D.BufferStyle = BufferStyle.DoubleBuffer

                'Add Unzoom button that appears only when needed
                UnzoomButton = New Button()
                With UnzoomButton
                    .Location = New Point(4, 4)
                    .Size = New Size(72, 22)
                    .Text = "Unzoom"
                    .Visible = False
                End With
                .Controls.Add(UnzoomButton)
            End With
        End Sub

        Private Sub Chart_Zoomed(sender As Object, e As EventArgs) Handles Me.Zoomed
            UnzoomButton.Visible = True
        End Sub

        Private Sub Chart_Unzoomed(sender As Object, e As EventArgs) Handles Me.UndoneZoom
            UnzoomButton.Visible = False
        End Sub

        Private Sub UnzoomButton_Click(sender As Object, e As EventArgs) Handles UnzoomButton.Click
            Me.Zoom.Undo()
        End Sub

    End Class
End Namespace